package fr.ippon.geohipster.web.rest;

import com.codahale.metrics.annotation.Timed;
import fr.ippon.geohipster.domain.Geocity;
import fr.ippon.geohipster.service.GeocityService;
import fr.ippon.geohipster.web.rest.util.HeaderUtil;
import fr.ippon.geohipster.web.rest.util.PaginationUtil;
import io.swagger.annotations.ApiParam;
import io.github.jhipster.web.util.ResponseUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.net.URI;
import java.net.URISyntaxException;

import java.util.List;
import java.util.Optional;

/**
 * REST controller for managing Geocity.
 */
@RestController
@RequestMapping("/api")
public class GeocityResource {

    private final Logger log = LoggerFactory.getLogger(GeocityResource.class);

    private static final String ENTITY_NAME = "geocity";

    private final GeocityService geocityService;

    public GeocityResource(GeocityService geocityService) {
        this.geocityService = geocityService;
    }

    /**
     * POST  /geocities : Create a new geocity.
     *
     * @param geocity the geocity to create
     * @return the ResponseEntity with status 201 (Created) and with body the new geocity, or with status 400 (Bad Request) if the geocity has already an ID
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PostMapping("/geocities")
    @Timed
    public ResponseEntity<Geocity> createGeocity(@RequestBody Geocity geocity) throws URISyntaxException {
        log.debug("REST request to save Geocity : {}", geocity);
        if (geocity.getId() != null) {
            return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "idexists", "A new geocity cannot already have an ID")).body(null);
        }
        Geocity result = geocityService.save(geocity);
        return ResponseEntity.created(new URI("/api/geocities/" + result.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(ENTITY_NAME, result.getId().toString()))
            .body(result);
    }

    /**
     * PUT  /geocities : Updates an existing geocity.
     *
     * @param geocity the geocity to update
     * @return the ResponseEntity with status 200 (OK) and with body the updated geocity,
     * or with status 400 (Bad Request) if the geocity is not valid,
     * or with status 500 (Internal Server Error) if the geocity couldn't be updated
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PutMapping("/geocities")
    @Timed
    public ResponseEntity<Geocity> updateGeocity(@RequestBody Geocity geocity) throws URISyntaxException {
        log.debug("REST request to update Geocity : {}", geocity);
        if (geocity.getId() == null) {
            return createGeocity(geocity);
        }
        Geocity result = geocityService.save(geocity);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(ENTITY_NAME, geocity.getId().toString()))
            .body(result);
    }

    /**
     * GET  /geocities : get all the geocities.
     *
     * @param pageable the pagination information
     * @return the ResponseEntity with status 200 (OK) and the list of geocities in body
     */
    @GetMapping("/geocities")
    @Timed
    public ResponseEntity<List<Geocity>> getAllGeocities(@ApiParam Pageable pageable) {
        log.debug("REST request to get a page of Geocities");
        Page<Geocity> page = geocityService.findAll(pageable);
        HttpHeaders headers = PaginationUtil.generatePaginationHttpHeaders(page, "/api/geocities");
        return new ResponseEntity<>(page.getContent(), headers, HttpStatus.OK);
    }

    /**
     * GET  /geocities/:id : get the "id" geocity.
     *
     * @param id the id of the geocity to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the geocity, or with status 404 (Not Found)
     */
    @GetMapping("/geocities/{id}")
    @Timed
    public ResponseEntity<Geocity> getGeocity(@PathVariable Long id) {
        log.debug("REST request to get Geocity : {}", id);
        Geocity geocity = geocityService.findOne(id);
        return ResponseUtil.wrapOrNotFound(Optional.ofNullable(geocity));
    }

    /**
     * DELETE  /geocities/:id : delete the "id" geocity.
     *
     * @param id the id of the geocity to delete
     * @return the ResponseEntity with status 200 (OK)
     */
    @DeleteMapping("/geocities/{id}")
    @Timed
    public ResponseEntity<Void> deleteGeocity(@PathVariable Long id) {
        log.debug("REST request to delete Geocity : {}", id);
        geocityService.delete(id);
        return ResponseEntity.ok().headers(HeaderUtil.createEntityDeletionAlert(ENTITY_NAME, id.toString())).build();
    }
}
